#!/bin/bash

config="/etc/jtb.conf"
if [ ! -f $config ]; then
	echo "Config file $config does not exists"
	exit
fi

function usage() {
	echo -e "Usage: $0 [OPTION]\n"
	echo -e "\t-s\t\tshow commit stats and exit"
	echo -e "\t-q\t\tquiet mode, no output if no errors occured"
	echo -e "\t-d date\t\tspecify date, e.g. 2013-03-02 (default: today)"
	echo -e "\t-t time\t\tspecify work start time, e.g. 11:57 (overrides config \"work_start_at\" value)"
	echo -e "\t-h\t\tdisplay this help and exit\n"
	echo -e "Examples:"
	echo -e "\t$0 -q"
	echo -e "\t$0 -s -d 2013-03-05"
	echo -e "\t$0 -d 2013-10-18 -t 10:05"
}

source $config

work_date=$(date "+%Y-%m-%d")

opt_quiet=0
opt_stat=0

while getopts ":hqsd:t:" opt;
do
	case $opt in
		d) work_date=$OPTARG;;
		t) work_start_at=$OPTARG;;
		q) opt_quiet=1;;
		s) opt_stat=1;;
		h) usage; exit;;
		?) echo -e "$0: invalid option -- '$OPTARG'\nTry '$0 -h' for more information."; exit;;
	esac
done

time_offset=0
master_task=0

declare -A tasks
declare -A times
declare -A sortedtasks

function calc_tasks() {

	local jira_key_length=$(expr ${#jira_key} + 1)

	for line in $mercurial_repositories;
	do
		repo=$(echo $line | tr -d ' ')

		if [ "$repo" != "" ]; then
			changesets=$(hg log -u $mercurial_user -M -d $work_date --template "{date|rfc822date}|{desc}\n" --repository $repo | tr " " "_")
			for changeset in $changesets;
			do
				timestamp=$(echo $changeset | cut -d "|" -f1 | tr "_" " ")
				timestamp=$(date -d "$timestamp" "+%s")

				task=$(echo $changeset | cut -d "|" -f2 | sed "s/.*$jira_key\-\([0-9]\+\).*/$jira_key\-\\1/")

				if [ "${task:0:$jira_key_length}" == "$jira_key-" ]; then
					tasks[$timestamp]=${task:$jira_key_length}
				fi
			done
		fi
	done
}

function calc_times() {

	local offset=0
	local total_time=0
	local longest_task=0
	local longest_time=0
	local prevtimestamp=$(date -d "$work_date $work_start_at" "+%s")
	
	sortedtasks=$(for ts in ${!tasks[@]}; do echo "$ts:${tasks[$ts]}"; done | sort -k1 -n)

	for taskhash in ${sortedtasks[@]}
	do
		timestamp=$(echo $taskhash | cut -d ":" -f1)
		task=${tasks[$timestamp]}

		time=$(expr $timestamp - $prevtimestamp)
		prevtimestamp=$timestamp

		times[$task]=$(expr ${times[$task]} + $time)
		total_time=$(expr $total_time + $time)
	done

	for task in ${!times[@]}
	do
		time=${times[$task]}

		if [ $(expr $time \> $longest_time) == 1 ]; then
			longest_time=$time
			longest_task=$task
		fi
	done

	# some strange logic
	if [ $total_time \> $work_duration ]; then
		current_total_time=$(expr $total_time - 3600)
		if [ $current_total_time \< $work_duration ]; then
			offset=$(expr $work_duration - $current_total_time - 3600)
		else
			offset=$(expr -3600)
		fi
	else
		offset=$(expr $work_duration - $total_time)
	fi

	if [ $longest_task \> 0 ]; then
		times[$longest_task]=$(expr ${times[$longest_task]} + $offset)

		time_offset=$offset
		master_task=$longest_task
	fi
}

function get_jira_time() {
	echo "$(expr $1 \/ 60)m"
}

function get_jira_task_id() {
	echo "$jira_key-$1"
}

function line() {
	printf "="%.0s {1..50}
	echo
}

function stat() {

	# calc real total time
	local total_time=0

	line
	echo "Calculated task commits:"
	for taskhash in ${sortedtasks[@]}
	do
		timestamp=$(echo $taskhash | cut -d ":" -f1)
		date=$(date -d "@$timestamp" "+%H:%M:%S, %d.%m.%Y")
		echo "$(get_jira_task_id ${tasks[$timestamp]}) at $date"
	done

	line
	echo "Calculated times:"
	for task in ${!times[@]}
	do
		offset=""
		time=${times[$task]}
		if [ $task == $master_task ]; then
			offset="(include $(get_jira_time $time_offset) offset)"
		fi
		echo "$(get_jira_task_id $task): $(get_jira_time $time) $offset"
		total_time=$(expr $total_time + $time)
	done

	echo -e "\nTotal: $(get_jira_time $total_time)"

	line
}

function send_time() {

	url="$jira_url/rest/api/2/issue/$1/worklog?adjustEstimate=new&newEstimate=0m"
	json="{\"comment\": \"timebot was here\", \"started\": \""$work_date"T22:30:00.000+0400\", \"timeSpent\": \"$(get_jira_time $2)\"}"
	lines=$(curl -s -u $jira_username:$jira_password -X POST -H "Content-Type: application/json" --data "$json" $url)

	if [ $opt_quiet == 0 ]; then
		result="success"
		line=$(echo ${lines[@]} | grep "\"id\":\"\([0-9]\+\)\"")
		if [ "$line" == "" ]; then
			result="error"
		fi
		echo "$1: $result"
	fi
}

function send_times() {
	for task in ${!times[@]}
	do
		send_time "$(get_jira_task_id $task)" "${times[$task]}"
	done
}

calc_tasks
calc_times

if [ $opt_stat == 1 ]; then
	stat
elif [ $opt_quiet == 1 ]; then
	send_times
else
	stat

	if [ ${#times[@]} \> 0 ]; then
		read -p "Do you want to send these times to jira? [y/n] "
		if [ "$REPLY" == "y" ]; then
			send_times
		fi
	fi
fi
